package com.mortals.xhx.module.home.web;

import cn.hutool.core.net.url.UrlBuilder;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.IUser;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.util.DateUtils;
import com.mortals.framework.util.HttpUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.framework.web.BaseJsonBodyController;
import com.mortals.xhx.common.code.NewsSourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.app.AppPdu;
import com.mortals.xhx.feign.app.IAppFeign;
import com.mortals.xhx.feign.base.IApiBaseManagerFeign;
import com.mortals.xhx.feign.base.pdu.DeptPdu;
import com.mortals.xhx.feign.base.pdu.SitePdu;
import com.mortals.xhx.feign.rsp.ApiResp;
import com.mortals.xhx.feign.site.ISiteFeign;
import com.mortals.xhx.module.baseset.model.BasesetEntity;
import com.mortals.xhx.module.baseset.model.BasesetQuery;
import com.mortals.xhx.module.baseset.service.BasesetService;
import com.mortals.xhx.module.device.model.DeviceMatterDatumQuery;
import com.mortals.xhx.module.device.service.DeviceMatterDatumService;
import com.mortals.xhx.module.home.pdu.HomeQueryPdu;
import com.mortals.xhx.module.hotword.model.HotwordEntity;
import com.mortals.xhx.module.hotword.model.HotwordQuery;
import com.mortals.xhx.module.hotword.service.HotwordService;
import com.mortals.xhx.module.matter.model.*;
import com.mortals.xhx.module.matter.service.MatterDatumPrintService;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import com.mortals.xhx.module.matter.service.MatterService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("home")
@Slf4j
public class HomeController extends BaseJsonBodyController {

    @Autowired
    private IAppFeign appFeign;
    @Autowired
    private IApiBaseManagerFeign apiBaseManagerFeign;
    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private MatterService matterService;
    @Autowired
    private HotwordService hotwordService;
    @Autowired
    private ISiteFeign siteFeign;
    @Autowired
    private BasesetService basesetService;
    @Autowired
    private MatterDatumPrintService matterDatumPrintService;
    @Autowired
    private DeviceMatterDatumService deviceMatterDatumService;


    @PostMapping({"site/list"})
    public Rest<Object> list() {
        IUser user = this.getCurUser();
        if (user == null) {
            throw new AppException("用户未登录");
        }
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询用户站点列表";

        int code = 1;
        try {
            if (StringUtils.isNotEmpty(user.getSiteIds())) {
                SitePdu sitePdu = new SitePdu();
                List<String> siteIds = Arrays.asList(user.getSiteIds().split(","));
                List<Long> idList = new ArrayList<>();
                siteIds.forEach(s -> {
                    idList.add(DataUtil.converStr2Long(s, 0));
                });
                sitePdu.setIdList(idList);
                String resp = apiBaseManagerFeign.getSitesByQuery(sitePdu);
                ApiResp<JSONObject> apiResp = JSON.parseObject(resp, ApiResp.class);
                if (apiResp.getCode() != YesNoEnum.YES.getValue()) {
                    throw new AppException("获取用户站点列表树数据失败：" + apiResp.getMsg());
                }
                if (apiResp.getData().get("data") != null) {
                    model.put("data", apiResp.getData().get("data"));
                } else {
                    model.put("data", Collections.emptyList());
                }
            } else {
                model.put("data", Collections.emptyList());
            }
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }


    @PostMapping({"depts"})
    @UnAuth
    public Rest<Object> depts(@RequestBody HomeQueryPdu queryPdu) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询站点部门";
        int code = VALUE_RESULT_SUCCESS;
        model.put("deptMap", new HashMap<>());
        try {
            if (queryPdu.getSiteId() != null) {
                if (!ObjectUtils.isEmpty(queryPdu.getDeviceCode())) {
                    Map<String, String> deptMap = deviceMatterDatumService.find(new DeviceMatterDatumQuery().siteId(queryPdu.getSiteId()).deviceCode(queryPdu.getDeviceCode())).stream().collect(Collectors.toMap(x -> x.getDeptCode(), y -> y.getDeptName(), (o, n) -> n));
                    ;
                    model.put("deptMap", deptMap);
                    // deptPdu.setIdList(deptIdList);
                } else {
                    Map<String, String> deptMap = matterService.find(new MatterQuery().siteId(queryPdu.getSiteId())).stream().collect(Collectors.toMap(x -> x.getDeptCode(), y -> y.getDeptName(), (o, n) -> n));
                    model.put("deptMap", deptMap);
                }
            }
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = VALUE_RESULT_FAILURE;
            this.doException(this.request, busiDesc, model, var9);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    @PostMapping({"dept/list"})
    @UnAuth
    public Rest<Object> deptList(@RequestBody HomeQueryPdu queryPdu) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询站点部门列表";
        int code = 1;
        try {
            if (queryPdu.getSiteId() != null) {
                DeptPdu deptPdu = new DeptPdu();
                deptPdu.setSiteId(queryPdu.getSiteId());
                deptPdu.setSize(-1);
                if (!ObjectUtils.isEmpty(queryPdu.getDeviceCode())) {
                    List<Long> deptIdList = deviceMatterDatumService.find(new DeviceMatterDatumQuery().deviceCode(queryPdu.getDeviceCode())).stream().map(item -> item.getDeptId()).collect(Collectors.toList());
                    deptPdu.setIdList(deptIdList);
                }
                String resp = apiBaseManagerFeign.getDeptByQuery(deptPdu);
                ApiResp<JSONObject> apiResp = JSON.parseObject(resp, ApiResp.class);
                if (apiResp.getCode() != YesNoEnum.YES.getValue()) {
                    throw new AppException("获取用户站点部门列表失败：" + apiResp.getMsg());
                }
                if (apiResp.getData().get("data") != null) {
                    model.put("data", apiResp.getData().get("data"));
                } else {
                    model.put("data", Collections.emptyList());
                }
            } else {
                model.put("data", Collections.emptyList());
            }
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    @PostMapping({"notice/list"})
    @UnAuth
    public String noteList(@RequestBody HomeQueryPdu queryPdu) {
        String url = GlobalSysInfo.getParamValue(Constant.PARAMS_NOTICE_URL, "http://192.168.0.98:8090/api/");
        UrlBuilder domain = UrlBuilder.of(url);
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询新闻列表";
        int code = VALUE_RESULT_SUCCESS;
        try {
            Map<String, String> params = new HashMap<>();
            params.put("siteid", queryPdu.getSiteId() + "");
            params.put("title", queryPdu.getTitle());
            params.put("type", queryPdu.getType() + "");
            params.put("page", queryPdu.getPage() + "");
            params.put("size", queryPdu.getSize() + "");
            UrlBuilder urlBuilder = domain.addPath("news").addPath("newsList");
            String response = HttpUtil.doGet(urlBuilder.toString(), params);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
            return response;
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(this.request, busiDesc, model, e);
        }
        ret.setCode(code);
        ret.setData(model);
        return JSONObject.toJSONString(ret);
    }


    @GetMapping({"notice/info"})
    @UnAuth
    public String noteInfo(Long id) {
        String url = GlobalSysInfo.getParamValue(Constant.PARAMS_NOTICE_URL, "http://192.168.0.98:8090/api/");
        UrlBuilder domain = UrlBuilder.of(url);
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询新闻详细";
        int code = VALUE_RESULT_SUCCESS;
        try {
            Map<String, String> params = new HashMap<>();
            params.put("id", id.toString());
            UrlBuilder urlBuilder = domain.addPath("news").addPath("newInfo");
            String response = HttpUtil.doGet(urlBuilder.toString(), params);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
            return response;
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(this.request, busiDesc, model, e);
        }

        ret.setCode(code);
        ret.setData(model);
        return JSONObject.toJSONString(ret);
    }

    /**
     * 合成表单与模板
     */
    @PostMapping(value = "mergeFormToDocx")
    @UnAuth
    public String mergeFormToDocx(@RequestBody MatterDatumEntity docFormVo) {
        JSONObject jsonObject = new JSONObject();
        try {
            String paths = matterDatumService.mergeFormToDoc(docFormVo, getContext());
            String[] vals = paths.split(";");
            if (vals.length > 1) {
                jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
                jsonObject.put("mergeDoc", vals[0]);
                jsonObject.put("preview", vals[1]);
            } else {
                throw new AppException("合成表单异常！");
            }

            //todo 保存表单数据
//            docFormVo.setCreateTime(new Date());
//            docFormVo.setCreateUserId(getContextUserId(getContext()));
//            docFormVo.setCreateUser("");
//            docFormService.save(docFormVo, getContext());
        } catch (Exception e) {
            log.error("合成表单异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }

    @PostMapping({"info"})
    @UnAuth
    public Rest<Object> homePageInfo(@RequestBody HomeQueryPdu homeQueryPdu) {
        log.info("homePageInfo:{}", JSON.toJSONString(homeQueryPdu));

        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        String busiDesc = "获取首页数据";
        int code = VALUE_RESULT_SUCCESS;
        try {
            HotwordQuery hotwordQuery = new HotwordQuery();
            hotwordQuery.siteId(homeQueryPdu.getSiteId());
            List<OrderCol> orderColList = new ArrayList<>();
            orderColList.add(new OrderCol("wordsSource", OrderCol.ASCENDING));
            orderColList.add(new OrderCol("searchCount", OrderCol.DESCENDING));
            hotwordQuery.setOrderColList(orderColList);

            PageInfo pageInfo = new PageInfo();
            int nums = GlobalSysInfo.getParamIntValue(Constant.PARAMS_HOTWORDS_DEFAULT_NUM, 10);
            pageInfo.setPrePageResult(nums);


            List<HotwordEntity> hotwordEntities = hotwordService.find(hotwordQuery, pageInfo, null).getList();
            model.put("hotWords", hotwordEntities);
            MatterQuery matterQuery = new MatterQuery();
            matterQuery.setSiteId(homeQueryPdu.getSiteId());
            int matterCont = matterService.count(matterQuery, this.getContext());
            matterQuery.setIsRecommend(YesNoEnum.YES.getValue());
            List<MatterEntity> matterList = matterService.find(matterQuery, this.getContext());
            model.put("matterList", matterList);   //热门事项
            model.put("matterCont", matterCont);   //入驻事项数量
            MatterDatumQuery matterDatumQuery = new MatterDatumQuery();
            matterDatumQuery.setSiteId(homeQueryPdu.getSiteId());
            int datumCont = matterDatumService.count(matterDatumQuery, this.getContext());
            matterDatumQuery.setIsRecommend(YesNoEnum.YES.getValue());
            List<MatterDatumEntity> datumList = matterDatumService.find(matterDatumQuery, this.getContext());
            model.put("datumList", datumList);    //热门表单
            model.put("datumCont", datumCont);    //入驻表单数量


            Map<Boolean, List<MatterDatumPrintEntity>> collect = matterDatumPrintService.find(new MatterDatumPrintQuery().siteId(homeQueryPdu.getSiteId()))
                    .stream().collect(Collectors.partitioningBy(x -> x.getType().equals(1)));
            List<MatterDatumPrintEntity> matterDatumPrintLocalEntities = collect.get(true);
            List<MatterDatumPrintEntity> matterDatumPrintOnlineEntities = collect.get(false);

            model.put("localPrint", matterDatumPrintLocalEntities.size());     //本地打印数量
            model.put("onlineSubmit", matterDatumPrintOnlineEntities.size());   //在线提交数量

            MatterDatumPrintQuery matterDatumPrintQuery = new MatterDatumPrintQuery();
            matterDatumPrintQuery.setSiteId(homeQueryPdu.getSiteId());
            matterDatumPrintQuery.setCreateTimeStart(DateUtils.getStrDate(new Date()));
            matterDatumPrintQuery.setCreateTimeEnd(DateUtils.getStrDate(new Date()));
            long todaySum = matterDatumPrintService.find(matterDatumPrintQuery).stream().collect(Collectors.summarizingInt(x -> x.getPrintPage())).getSum();
            model.put("dayThrift", todaySum);      //今日节约
            long sum = matterDatumPrintOnlineEntities.stream().collect(Collectors.summarizingInt(x -> x.getPrintPage())).getSum();

            model.put("totalThrift", sum * 2);    //累计节约
            Rest<com.mortals.xhx.common.pdu.site.SitePdu> info = siteFeign.info(homeQueryPdu.getSiteId());

            model.put("title", info.getData() == null ? "" : info.getData().getSiteName()); //标题

            BasesetEntity basesetEntity = basesetService.selectOne(new BasesetQuery().siteId(homeQueryPdu.getSiteId()));

            model.put("blankCount", basesetEntity == null ? 20 : basesetEntity.getPrintDisplay()); //空白样表数量
            model.put("newsSource", basesetEntity == null ? 1 : basesetEntity.getNewsSource()); //新闻来源
            model.put("message_info", busiDesc + "成功");
            this.addDict(model, "newsSource", NewsSourceEnum.getEnumMap());
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = VALUE_RESULT_FAILURE;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
      //  log.info("home resp:{}", JSON.toJSONString(ret));
        return ret;
    }

    /**
     * 自助服务应用
     *
     * @return
     */
    @GetMapping({"app/list"})
    @UnAuth
    public String appList(Long siteId) {
        String serverName = request.getHeader("server-name");
        Integer serverPort = DataUtil.converStr2Int(request.getHeader("server-port"), 0);

        //log.info("【应用请求】【请求体】-->serverName{} ,port:{}", serverName, serverPort);
        String appWhiteStr = GlobalSysInfo.getParamValue(Constant.PARAMS_WHITE_APP_LIST, "中心简介,办事指南,意见建议,通知公告");

        log.info("【应用请求】【请求体】-->appWhiteStr{} ", appWhiteStr);
        Set<String> appWhiteSet = StrUtil.split(appWhiteStr, ",").stream().collect(Collectors.toSet());
        Rest<List<AppPdu>> ret = new Rest();
        List<AppPdu> collect = new ArrayList<>();
        int code = VALUE_RESULT_SUCCESS;
        try {
            AppPdu appPdu = new AppPdu();
            appPdu.setSiteId(siteId);
            appPdu.setSize(-1);
            Rest<RespData<List<AppPdu>>> appRest = appFeign.list(appPdu);

            String domainUrl = UrlBuilder.ofHttp(serverName).setPort(serverPort > 0 ? serverPort : 11078).build();
          ///  String domainUrl1 = StrUtil.sub(domainUrl, 0, domainUrl.length());

            log.info("appRest:" + JSON.toJSONString(appRest));
            log.info("domainUrl:" + domainUrl);
            if (appRest.getCode() == YesNoEnum.YES.getValue()) {
                collect = appRest.getData().getData().stream().filter(f -> appWhiteSet.contains(f.getAppName()))
                        .peek(item -> {
                            item.setCustUrl(UrlBuilder.ofHttp(domainUrl).addPath(item.getCustUrl()).build());
                            //item.setAppIconPath(UrlBuilder.ofHttp(domainUrl1).addPath(item.getAppIconPath()).build());
                          //  log.info(item.getCustUrl());
                           // log.info(item.getAppIconPath());
                        }).collect(Collectors.toList());
            }
        } catch (Exception e) {
            log.error("异常",e);
            code = VALUE_RESULT_FAILURE;
        }
        ret.setCode(code);
        ret.setData(collect);
        return JSONObject.toJSONString(ret);
    }

}
