package com.mortals.xhx.module.matter.service.impl;

import cn.hutool.core.util.IdUtil;
import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.utils.WordUtil;
import com.mortals.xhx.module.device.model.DeviceEntity;
import com.mortals.xhx.module.device.model.DeviceQuery;
import com.mortals.xhx.module.device.service.DeviceService;
import com.mortals.xhx.module.matter.model.MatterDatumEntity;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import com.mortals.xhx.module.matter.service.MatterService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.matter.dao.MatterDatumPrintDao;
import com.mortals.xhx.module.matter.model.MatterDatumPrintEntity;
import com.mortals.xhx.module.matter.service.MatterDatumPrintService;
import org.springframework.util.ObjectUtils;

import java.util.List;

/**
 * MatterDatumPrintService
 * 打印提交 service实现
 *
 * @author zxfei
 * @date 2022-12-09
 */
@Service("matterDatumPrintService")
public class MatterDatumPrintServiceImpl extends AbstractCRUDServiceImpl<MatterDatumPrintDao, MatterDatumPrintEntity, Long> implements MatterDatumPrintService {

    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private MatterService matterService;
    @Autowired
    private DeviceService deviceService;

    @Value("${upload.path}")
    private String filePath;

    @Override
    protected void saveBefore(MatterDatumPrintEntity entity, Context context) throws AppException {
        // MatterDatumEntity matterDatumEntity = matterDatumService.get(entity.getMaterialId());
        if (!ObjectUtils.isEmpty(entity.getMatterId())) {
            MatterEntity matterEntity = matterService.get(entity.getMatterId());
            entity.setMatterName(matterEntity == null ? "" : matterEntity.getMatterName());
            entity.setMatterCode(matterEntity == null ? "" : matterEntity.getMatterNo());
        }

        if (!ObjectUtils.isEmpty(entity.getMaterialId())) {
            MatterDatumEntity matterDatumEntity = matterDatumService.get(entity.getMaterialId());
            if (!ObjectUtils.isEmpty(matterDatumEntity)) {
                entity.setMatterId(matterDatumEntity.getMatterId());
                entity.setMaterialName(matterDatumEntity.getMaterialName());
            }
        }
        entity.setOrderId(IdUtil.simpleUUID());
        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
       // Integer pageByDoc = WordUtil.getPageByDoc(rootPath + entity.getDocPath());
        entity.setPrintPage(1);

        if (!ObjectUtils.isEmpty(entity.getDeviceCode())) {
            DeviceEntity deviceEntity = deviceService.selectOne(new DeviceQuery().deviceCode(entity.getDeviceCode()));
            entity.setDeviceName(deviceEntity == null ? "" : deviceEntity.getDeviceName());
        }


        super.saveBefore(entity, context);
    }


    @Override
    protected void findAfter(MatterDatumPrintEntity params, PageInfo pageInfo, Context context, List<MatterDatumPrintEntity> list) throws AppException {
        super.findAfter(params, pageInfo, context, list);

        list.stream().forEach(entity -> {
            MatterEntity matterEntity = matterService.get(entity.getMatterId(), context);
            if (!ObjectUtils.isEmpty(matterEntity)) {
                entity.setMatterFullName(matterEntity.getMatterName());
                entity.setMateriaFullName(entity.getMaterialName());
            }
        });

    }
}