package com.mortals.xhx.module.matter.service.impl;

import cn.hutool.core.util.StrUtil;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.MatterSourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.ParamKey;
import com.mortals.xhx.common.pdu.site.SitePdu;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.feign.site.ISiteFeign;
import com.mortals.xhx.module.hotword.model.HotwordEntity;
import com.mortals.xhx.module.hotword.model.HotwordQuery;
import com.mortals.xhx.module.hotword.service.HotwordService;
import com.mortals.xhx.module.matter.model.MatterDatumEntity;
import com.mortals.xhx.module.matter.model.MatterDatumQuery;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import com.mortals.xhx.module.sheet.dao.SheetMatterDao;
import com.mortals.xhx.module.sheet.model.SheetMatterEntity;
import com.mortals.xhx.module.sheet.service.SheetMatterService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.matter.dao.MatterDao;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.service.MatterService;
import org.springframework.util.ObjectUtils;

import java.util.*;

/**
 * MatterService
 * 事项申请材料 service实现
 *
 * @author zxfei
 * @date 2022-09-27
 */
@Service("matterService")
public class MatterServiceImpl extends AbstractCRUDServiceImpl<MatterDao, MatterEntity, Long> implements MatterService {

    private static int RECOMMEND_COUNT = 5;
    @Autowired
    private SheetMatterService sheetMatterService;
    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private ISiteFeign siteFeign;
    @Autowired
    private HotwordService hotwordService;

    @Override
    protected void findAfter(MatterEntity entity, PageInfo pageInfo, Context context, List<MatterEntity> list) throws AppException {
        for (MatterEntity matterEntity : list) {
            matterEntity.setDatumCount(matterEntity.getMatterDatumList().size());
            matterEntity.setMatterFullName(matterEntity.getMatterName());
            matterEntity.getMatterDatumList().forEach(matterDatumEntity -> {
                matterDatumEntity.setMatterName(matterEntity.getMatterName());
                matterDatumEntity.setMatterNo(matterEntity.getMatterNo());
                matterDatumEntity.setDeptCode(matterEntity.getDeptCode());
                matterDatumEntity.setMateriaFullName(matterDatumEntity.getMateriaFullName());
            });

            if (entity.getIsTerminal() == YesNoEnum.YES.getValue()) {
                String searchMatterName = StrUtil.subBetween(entity.getMatterFullName(), "%");
                if (!ObjectUtils.isEmpty(searchMatterName)) {
                    //终端访问，标红查询参数
                    String str = "<span style=\"color:red;font-weight:bold\">%s</span>";
                    String replaceStr = StrUtil.replace(matterEntity.getMatterName(), searchMatterName, String.format(str, searchMatterName));
                    matterEntity.setMatterFullName(replaceStr);
                }

            }
        }
        if (!ObjectUtils.isEmpty(list)&&!ObjectUtils.isEmpty(entity.getMatterFullName())) {
            //添加更新热门词汇
            HotwordEntity hotwordEntity = hotwordService.selectOne(new HotwordQuery().siteId(entity.getSiteId()).hotwords(entity.getMatterFullName()), context);
            if (ObjectUtils.isEmpty(hotwordEntity)) {
                //新增
                hotwordEntity = new HotwordEntity();
                hotwordEntity.initAttrValue();
                hotwordEntity.setHotwords(StrUtil.subBetween(entity.getMatterFullName(), "%"));
                hotwordEntity.setSearchCount(1);
                hotwordEntity.setSiteId(entity.getSiteId());
                hotwordService.save(hotwordEntity, context);
            } else {
                //更新
                hotwordEntity.setSearchCount(1 + hotwordEntity.getSearchCount());
                hotwordService.update(hotwordEntity, context);
            }
        }
    }

    @Override
    public Rest<String> createMatterbBySheetMatter(Long[] sheetMatterIds, Long siteId, Context context) {
        if (ObjectUtils.isEmpty(sheetMatterIds)) {
            throw new AppException("事项ids is null");
        }
        int success = 0;
        int fail = 0;
        List<MatterEntity> matterEntities = new ArrayList<>();
        List<SheetMatterEntity> sheetMatterEntityList = sheetMatterService.get(sheetMatterIds, context);
        for (SheetMatterEntity sheetMatterEntity : sheetMatterEntityList) {
            MatterEntity matterEntity = updateOrSave(sheetMatterEntity, siteId, context);
            if (!ObjectUtils.isEmpty(matterEntity)) {
                matterEntities.add(matterEntity);
                success++;
            } else {
                fail++;
            }
        }
        if (!ObjectUtils.isEmpty(matterEntities)) {
            this.save(matterEntities, context);
        }
        String msg = "当前加入事项已存在！";
        if (sheetMatterIds.length == 1) {
            if (success > 0) {
                msg = "加入事项成功！";
            }
        } else if (sheetMatterIds.length > 1) {
            if (success > 0 && fail == 0) {
                msg = String.format("加入事项成功%d条!", success);
            } else if (success > 0 && fail > 0) {
                msg = String.format("加入事项成功%d条，重复加入事项%d条！", success, fail);
            } else if (success == 0 && fail > 0) {
                msg = String.format("重复加入事项%d条！", fail);
            }
        }
        return Rest.ok(msg);
    }

    private MatterEntity updateOrSave(SheetMatterEntity sheetMatterEntity, Long siteId, Context context) {
        int count = this.count(new MatterQuery().siteId(siteId).matterNo(sheetMatterEntity.getMatterNo()), context);
        if (count == 0) {
            MatterEntity matterEntity = new MatterEntity();
            matterEntity.initAttrValue();
            matterEntity.setSiteId(siteId);
            matterEntity.setDeptCode(sheetMatterEntity.getDeptCode());
            matterEntity.setDeptName(sheetMatterEntity.getDeptName());
            matterEntity.setTid(sheetMatterEntity.getTid());
            matterEntity.setTcode(sheetMatterEntity.getTcode());
            matterEntity.setTname(sheetMatterEntity.getTname());
            matterEntity.setMatterName(sheetMatterEntity.getMatterName());
            matterEntity.setEnglishName(sheetMatterEntity.getEnglishName());
            matterEntity.setMatterNo(sheetMatterEntity.getMatterNo());
            matterEntity.setMatterFullName(sheetMatterEntity.getMatterName());
            matterEntity.setSource(MatterSourceEnum.AUTO.getValue());
            matterEntity.setCreateUserId(this.getContextUserId(context));
            matterEntity.setCreateTime(new Date());
            return matterEntity;
        }
        return null;
    }

    @Override
    public Rest<String> recommend(Long id, Context context) {
        MatterEntity matterEntity = this.get(id, context);
        if (matterEntity == null) {
            throw new AppException("数据已更改,请重试");
        }
        String msg = "推荐成功！";
        int recommendCount = this.count(new MatterQuery().siteId(matterEntity.getSiteId()).isRecommend(YesNoEnum.YES.getValue()), context);
        if (matterEntity.getIsRecommend() == YesNoEnum.NO.getValue()) {
            if (recommendCount >= RECOMMEND_COUNT) {
                throw new AppException("超过推荐个数!");
            }
            matterEntity.setIsRecommend(YesNoEnum.YES.getValue());
        } else {
            matterEntity.setIsRecommend(YesNoEnum.NO.getValue());
            msg = "取消推荐成功！";
        }
        this.update(matterEntity, context);
        return Rest.ok(msg);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<MatterDatumEntity> matterDatumlist = matterDatumService.find(new MatterDatumQuery().matterIdList(Arrays.asList(ids)));
        matterDatumService.removeList(matterDatumlist, context);
        super.removeAfter(ids, context, result);
    }


}