package com.mortals.xhx.module.matter.web;

import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.key.ParamKey;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.common.utils.MemoryPagination;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.baseset.model.BasesetEntity;
import com.mortals.xhx.module.baseset.model.BasesetQuery;
import com.mortals.xhx.module.baseset.service.BasesetService;
import com.mortals.xhx.module.device.model.DeviceMatterDatumQuery;
import com.mortals.xhx.module.device.service.DeviceMatterDatumService;
import com.mortals.xhx.module.hotword.model.HotwordEntity;
import com.mortals.xhx.module.hotword.model.HotwordQuery;
import com.mortals.xhx.module.hotword.service.HotwordService;
import com.mortals.xhx.module.matter.model.MatterDatumInfo;
import com.mortals.xhx.module.matter.model.MatterDatumQuery;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.matter.model.MatterDatumEntity;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import org.apache.commons.lang3.ArrayUtils;

import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

/**
 * 事项申请材料
 *
 * @author zxfei
 * @date 2022-09-27
 */
@RestController
@RequestMapping("matter/datum")
public class MatterDatumController extends BaseCRUDJsonBodyMappingController<MatterDatumService, MatterDatumEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private BasesetService basesetService;
    @Autowired
    private DeviceMatterDatumService deviceMatterDatumService;

    public MatterDatumController() {
        super.setModuleDesc("事项申请材料");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "source", paramService.getParamBySecondOrganize("MatterDatum", "source"));
        this.addDict(model, "isRecommend", paramService.getParamBySecondOrganize("MatterDatum", "isRecommend"));
        super.init(model, context);
    }

    @Override
    protected void doListBefore(MatterDatumEntity query, Map<String, Object> model, Context context) throws AppException {
        List<OrderCol> orderColList = new ArrayList<>();
        orderColList.add(new OrderCol("isRecommend", OrderCol.DESCENDING));
        orderColList.add(new OrderCol("sort", OrderCol.DESCENDING));
        orderColList.add(new OrderCol("createTime", OrderCol.DESCENDING));
        query.setOrderColList(orderColList);
        super.doListBefore(query, model, context);
    }


    @RequestMapping(value = {"addPubdatum"}, method = {RequestMethod.POST, RequestMethod.GET})
    public String addPubdatum(Long[] ids, Long matterId, Long siteId) {
        Map<String, Object> model = new HashMap();
        int code = VALUE_RESULT_SUCCESS;
        String busiDesc = "添加公共库";
        try {
            Rest<String> rest = this.service.addPubdatum(ids, matterId, siteId, getContext());
            model.put(MESSAGE_INFO, rest.getMsg());

            this.recordSysLog(this.request, rest.getMsg());
        } catch (Exception var7) {
            code = VALUE_RESULT_FAILURE;
            this.doException(this.request, busiDesc, model, var7);
        }

        JSONObject ret = new JSONObject();
        ret.put("code", Integer.valueOf(code));
        ret.put("msg", model.remove(MESSAGE_INFO));
        ret.put("data", model);
        return ret.toJSONString();
    }

    @RequestMapping(value = {"recommend"}, method = {RequestMethod.POST, RequestMethod.GET})
    public String recommend(Long id, Long siteId) {
        Map<String, Object> model = new HashMap();
        int code = VALUE_RESULT_SUCCESS;
        String busiDesc = "推荐or取消推荐" + this.getModuleDesc();
        try {
            Rest<String> rest = this.service.recommend(id, siteId, getContext());
            model.put("message_info", rest.getMsg());
            this.recordSysLog(this.request, rest.getMsg());
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(this.request, busiDesc, model, e);
        }

        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }


    //空白样表查询
    @PostMapping(value = "blanklist")
    @UnAuth
    public Rest<Object> list(@RequestBody MatterDatumQuery query) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "查询空白样表";
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (StringUtils.isNotEmpty(query.getMateriaFullName())) {
                query.setMateriaFullName("%".concat(query.getMateriaFullName()).concat("%"));
            }
            BasesetEntity basesetEntity = basesetService.selectOne(new BasesetQuery().siteId(query.getSiteId()));
            int blankCount = basesetEntity == null ? 20 : basesetEntity.getPrintDisplay(); //空白样表数量
            PageInfo pageInfo = new PageInfo();
            pageInfo.setPrePageResult(blankCount);
            Result<MatterDatumEntity> result = this.service.find(query, pageInfo, context);
            //内存分页
            List<MatterDatumEntity> pagination = MemoryPagination.pagination(result.getList(), query.getPage(), query.getSize());
            List<MatterDatumInfo> collect = pagination.stream().map(item -> {
                MatterDatumInfo matterDatumInfo = new MatterDatumInfo();
                matterDatumInfo.setMaterialId(item.getId());
                BeanUtils.copyProperties(item, matterDatumInfo, BeanUtil.getNullPropertyNames(item));
                return matterDatumInfo;
            }).collect(Collectors.toList());

            if (!ObjectUtils.isEmpty(query.getDeviceCode())) {
                DeviceMatterDatumQuery deviceMatterDatumQuery = new DeviceMatterDatumQuery().deviceCode(query.getDeviceCode());
                List<Long> matterIdList = deviceMatterDatumService.find(deviceMatterDatumQuery, pageInfo, context).getList().stream().map(item -> item.getMatterId()).collect(Collectors.toList());

                if (!ObjectUtils.isEmpty(matterIdList)) {
                    MatterDatumQuery matterDatumQuery = new MatterDatumQuery();
                    matterDatumQuery.setMatterIdList(matterIdList);

                    Result<MatterDatumEntity> matterDatumEntityResult = this.service.find(matterDatumQuery, pageInfo, context);
                    pagination = MemoryPagination.pagination(matterDatumEntityResult.getList(), query.getPage(), query.getSize());
                    collect = pagination.stream().map(item -> {
                        MatterDatumInfo matterDatumInfo = new MatterDatumInfo();
                        matterDatumInfo.setMaterialId(item.getId());
                        BeanUtils.copyProperties(item, matterDatumInfo, BeanUtil.getNullPropertyNames(item));
                        return matterDatumInfo;
                    }).collect(Collectors.toList());
                }
/*                Set<Long> matterSet = deviceMatterDatumService.find(deviceMatterDatumQuery).stream().map(item -> item.getMatterId()).collect(Collectors.toSet());
                Iterator<MatterDatumInfo> iterator = collect.iterator();
                while (iterator.hasNext()) {
                    MatterDatumInfo next =  iterator.next();
                    if(!matterSet.contains(next.getMatterId())){
                        iterator.remove();
                    }
                }*/
            }
            model.put(KEY_RESULT_DATA, collect);
            model.put(PAGEINFO_KEY, result.getPageInfo());
            model.put(MESSAGE_INFO, busiDesc + "成功");
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get(KEY_RESULT_DICT));
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret;
    }

}