package com.mortals.xhx.module.skin.web;

import com.alibaba.fastjson.JSON;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.pdu.device.DeviceMsgReqPdu;
import com.mortals.xhx.feign.device.IDeviceMessageFeign;
import com.mortals.xhx.module.skin.model.SkinEntity;
import com.mortals.xhx.module.skin.service.SkinService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
*
* 皮肤
*
* @author zxfei
* @date 2023-04-03
*/
@RestController
@RequestMapping("skin")
@Slf4j
public class SkinController extends BaseCRUDJsonBodyMappingController<SkinService,SkinEntity,Long> {

    @Autowired
    private ParamService paramService;

    @Autowired
    private IDeviceMessageFeign deviceMessageFeign;

    public SkinController(){
        super.setModuleDesc( "皮肤");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "imageResolution", paramService.getParamBySecondOrganize("Skin","imageResolution"));
        this.addDict(model, "used", paramService.getParamBySecondOrganize("Skin","used"));
        super.init(model, context);
    }


    @Override
    protected int saveAfter(SkinEntity entity, Map<String, Object> model, Context context) throws AppException {
        //推送皮肤更新消息
        DeviceMsgReqPdu deviceMsgReqPdu = new DeviceMsgReqPdu();
        deviceMsgReqPdu.setSiteid(entity.getSiteId());
        deviceMsgReqPdu.setProductCode("tdj");
        deviceMsgReqPdu.setMessageType("edition");
        deviceMsgReqPdu.setData("W10=");
        deviceMsgReqPdu.setTimestamp(System.currentTimeMillis());
        Rest<String> rest = deviceMessageFeign.callbackByProduct(deviceMsgReqPdu);
        log.info("send device message==>{}", JSON.toJSONString(rest));

        return super.saveAfter(entity, model, context);
    }


    @PostMapping({"refreshSkin"})
    @UnAuth
    public Rest<Object> refreshSkin() {
        log.info("刷新皮肤");
        this.service.syncSkin();
        return Rest.ok("皮肤刷新操作成功");
    }
}